﻿;
; Screensaver to replace the "Diashow" Screensaver in Windows XP
; By Sascha 'SieGeL' Pfalz
; Written 2011 by IN-Ovation Software
; This is all GPL code, feel free to do what you want with it, but please keep copyright notices intact.
;
Global VERSION$ = "0.1";

Global NewList PictureFiles.s();

Global PREFS_IMAGE_PATH.s;
Global PREFS_LOG_FILE.s;
Global PREFS_DELAY.l;
;
; Include all other sources. Remember that global must be declared BEFORE the procedures
;
IncludeFile "functions.pbi"
IncludeFile "Dia-Show_config.pbi"
IncludeFile "RotateImage.pb"

#DELAY_STEP = 200
;
; Prototypes 
;
Declare Diashow()

; Activate Image decoder and setup all important sub-systems
UseJPEGImageDecoder();
UsePNGImageDecoder();
UseTIFFImageDecoder();
If InitKeyboard()=0 Or InitMouse()=0 Or InitSprite()=0
  MessageRequester("ERROR!","DirectX not found!",#MB_ICONWARNING)
  End
EndIf
;
; Get command line parameter. Windows sends /p for preview and /c for config window
;
Define FirstParam.s    =ProgramParameter() 
Define command.s       =LCase(Left(ReplaceString(FirstParam,"-","/"),2)) 
Define ParentWindow.l  =Val(StringField(FirstParam,2,":")) | Val(ProgramParameter()) 
;
; Try to read our preferences, fill in the defaults if no file exists yet
;
ReadPrefs();

;MessageRequester("Info","IPATH="+PREFS_IMAGE_PATH.s+Chr(13)+"LOGPATH="+PREFS_LOG_FILE.s+Chr(13)+"DELAY="+Str(PREFS_DELAY));
;End;

Select command      
   Case "/p" 
     ;Preview(ParentWindow) 
   Case "/c" 
      ConfigWindow()      
   Default 
      If PREFS_IMAGE_PATH = ""
        ConfigWindow()
      EndIf
      Diashow() 
EndSelect 
End;

;******************************************************************************
; Main procedure renders the pictures on-screen
;******************************************************************************
Procedure Diashow()
  Define rc,ScreenWidth,ScreenHeight,Quit,Screen,UsePicture,MaxItems,OrgImage,orientation,Image,ImageX,ImageY,OutX,OutY,ScalX,ScalY,PosX,PosY;
  ; Read list of available pictures, select the first one and load it:
  rc = ReadRecDir(PREFS_IMAGE_PATH.s,"*.*",PictureFiles.s());
  If rc <> #True
    ProcedureReturn 1
  EndIf
  MaxItems    = ListSize(PictureFiles.s());
  WriteToLog(PREFS_LOG_FILE,"Found "+Str(MaxItems)+" Pictures in path "+PREFS_IMAGE_PATH.s);

  If MaxItems <= 0
    MessageRequester("Warning!","No files found in directory "+PREFS_IMAGE_PATH+" ! ("+Str(MaxItems)+")");
    ProcedureReturn 1  
  EndIf
  ;
  ; Determine size of desktop, and open a 32-bit screen on that size  
  ;
  ExamineDesktops();
  ScreenWidth   = DesktopWidth(0);
  ScreenHeight  = DesktopHeight(0);
  
  Screen = OpenScreen(ScreenWidth,ScreenHeight,32,"Dia-Show",#PB_Screen_NoSynchronization);
  If Screen = 0
    MessageRequester("ERROR","Unable to open screen!!!");
    ProcedureReturn 1
  EndIf
  ;
  ; Determine the "Output" of the screen, we need this to actually draw the image on screen
  ;
  Quit = 0;
  Repeat
    UsePicture  = Random(MaxItems-1); 
    SelectElement(PictureFiles.s(),UsePicture);
    WriteToLog(PREFS_LOG_FILE,"Loading file "+PictureFiles.s())
    fsize = FileSize(PictureFiles.s());
    If fsize < 12
      WriteToLog(PREFS_LOG_FILE,"ERROR: File is less than 12 bytes?");
      Continue
    EndIf
    orientation = GetRotationExif(PictureFiles.s())
    OrgImage = LoadImage(#PB_Any,PictureFiles.s());
    If OrgImage = 0
      WriteToLog(PREFS_LOG_FILE,"Unable to load picture "+PictureFiles.s()+" !")
      Continue
    EndIf
    If orientation = 8
      Image = RotateImage(OrgImage,90);
      WriteToLog(PREFS_LOG_FILE,"RotateImage(): Orientation="+Str(orientation) + "| Name="+PictureFiles.s())
    ElseIf orientation = 6
      Image = RotateImage(OrgImage,270);
      WriteToLog(PREFS_LOG_FILE,"RotateImage(): Orientation="+Str(orientation) + "| Name="+PictureFiles.s())
    Else  
      Image = OrgImage;
    EndIf
    ; Determine the Size of the Image:
    ImageX = ImageWidth(Image);
    ImageY = ImageHeight(Image);
    ;
    ; Check if picture fits on screen. If this is not the case, we have to resize it on-the-fly
    ; Also we have to calculate the middle of the screen, so the picture is always fitted on screen
    ;
    ClearScreen(#Black)
    StartDrawing(ScreenOutput());
    
    OutX = ImageX;
    OutY = ImageY;

    If ImageX > ScreenWidth Or ImageY > ScreenHeight
    
      ScalX = ScreenWidth * 100 / ImageX;
      ScalY = ScreenHeight * 100 / ImageY;
      If ScalX > ScalY
        OutX = (ImageX * ScalY) / 100
        OutY = (ImageY * ScalY) / 100
      Else
        OutX = (ImageX * ScalX) / 100
        OutY = (ImageY * ScalX) / 100
      EndIf
    EndIf
    PosX = (ScreenWidth - OutX) / 2
    PosY = (ScreenHeight - OutY) / 2;
    DrawImage(ImageID(Image),PosX,PosY,OutX,OutY);
    FreeImage(Image);
    If IsImage(OrgImage)
      FreeImage(OrgImage);
    EndIf
    StopDrawing();
    FlipBuffers();
    If IsScreenActive() = 0
      WriteToLog(PREFS_LOG_FILE,"Screen is no longer active?");
      CloseScreen();
      ProcedureReturn 2
    EndIf
    Define i
    Define Delay = PREFS_DELAY * 1000
    For i = 0 To Delay Step #DELAY_STEP
      ExamineMouse()
      ExamineKeyboard()
      If KeyboardReleased(#PB_Key_All) Or KeyboardPushed(#PB_Key_All) Or MouseDeltaX() Or MouseDeltaY() Or MouseButton(1) Or MouseButton(2) 
        WriteToLog(PREFS_LOG_FILE,"Mouse or Keyboard Event found!");
        CloseScreen()
        ProcedureReturn 0
      EndIf
      Delay(#DELAY_STEP);
    Next
    ExamineMouse() : ExamineKeyboard() 
  Until KeyboardPushed(#PB_Key_All) Or KeyboardReleased(#PB_Key_All) Or MouseButton(1) Or MouseButton(2) Or MouseDeltaX() Or MouseDeltaY()
  CloseScreen()
  WriteToLog(PREFS_LOG_FILE,"Normal exit");
  ProcedureReturn 0
EndProcedure


; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 176
; FirstLine = 121
; Folding = -
; EnableAsm
; EnableUnicode
; EnableThread
; EnableXP
; EnableOnError
; UseIcon = ..\IN-Ovation_Software.ico
; Executable = Dia-Show.scr.exe
; EnableCompileCount = 446
; EnableBuildCount = 17
; EnableExeConstant
; IncludeVersionInfo
; VersionField0 = 0,1,%BUILDCOUNT,%COMPILECOUNT
; VersionField1 = 0,1,%BUILDCOUNT,%COMPILECOUNT
; VersionField2 = IN-Ovation Software
; VersionField3 = Dia-Show
; VersionField4 = 0.1
; VersionField5 = 0.1
; VersionField6 = Dia-Show Bildschirmschoner
; VersionField7 = Dia-Show
; VersionField8 = %SOURCE
; VersionField9 = GPL
; VersionField13 = webmaster@in-ovation.de
; VersionField14 = http://www.in-ovation.de