﻿;
; Config related stuff for Dia-Show screen saver
; 
Declare ReadPrefs();
Declare SavePrefs();
Declare ConfigWindow();

#MENU_ABOUT   = 1
#MENU_CLOSE   = 2
#MENU_WEBSITE = 3

#DIASHOW_IMAGE_PATH     = 10
#DIASHOW_CHOOSE_PATH    = 11
#DIASHOW_DELAY          = 12
#DIASHOW_DELAY_OUTPUT   = 13
#DIASHOW_LOGFILE        = 14
#DIASHOW_CHOOSE_LOGFILE = 15
#DIASHOW_BTN_SAVEPREFS  = 16
#DIASHOW_BTN_ABORT      = 17
#DIASHOW_BTN_COUNT      = 18

;******************************************************************************
; Tries to read the preferences, if no preferences are found, fill in some
; defaults
;******************************************************************************

Procedure ReadPrefs()
  Shared PREFS_DELAY.l,PREFS_IMAGE_PATH.s,PREFS_LOG_FILE.s;
  Define Homedir$,prefsFile$,prefs;  
  Homedir$ = GetHomeDirectory();
  prefsFile$ = HomeDir$ + "Dia-show.prefs";
  prefs = OpenPreferences(prefsFile$)
  PREFS_IMAGE_PATH.s  = ReadPreferenceString("image_path","");
  PREFS_LOG_FILE.s    = ReadPreferenceString("log_file","");
  PREFS_DELAY.l       = ReadPreferenceInteger("delay", 10);
  ClosePreferences();
EndProcedure

;******************************************************************************
; Saves the user preferences
;******************************************************************************

Procedure SavePrefs()
  Shared PREFS_DELAY.l,PREFS_IMAGE_PATH.s,PREFS_LOG_FILE.s;
  Define Homedir$,prefsFile$,prefs;  
  PREFS_IMAGE_PATH.s  = GetGadgetText(#DIASHOW_IMAGE_PATH);
  PREFS_LOG_FILE.s    = GetGadgetText(#DIASHOW_LOGFILE);
  PREFS_DELAY.l       = GetGadgetState(#DIASHOW_DELAY);
  Homedir$ = GetHomeDirectory();
  prefsFile$ = HomeDir$ + "Dia-show.prefs";
  prefs = OpenPreferences(prefsFile$)
  WritePreferenceString("image_path",PREFS_IMAGE_PATH.s);
  WritePreferenceString("log_file",PREFS_LOG_FILE.s);
  WritePreferenceInteger("delay",PREFS_DELAY.l);
  ClosePreferences()
EndProcedure

;******************************************************************************
; Configuration window
;******************************************************************************
Procedure ConfigWindow()
  Define EventID,Quit,MenuID,CompileInfo$,FPATH$,rc,MaxItems;
  NewList DummyList.s();
  
  If OpenWindow(0, 100, 100, 540, 240, "Dia-Show v"+VERSION$+" von Sascha 'SieGeL' Pfalz", #PB_Window_SystemMenu | #PB_Window_SizeGadget | #PB_Window_ScreenCentered)
    If CreateMenu(0, WindowID(0))    ; hier beginnt das Erstellen des Menüs...
      MenuTitle("Projekt")
      MenuItem(#MENU_ABOUT  , "Über..." + Chr(9) + "STRG+A")
      MenuItem(#MENU_WEBSITE, "Homepage..." + Chr(9) + "STRG-H");
      MenuItem(#MENU_CLOSE  , "Schließen" + Chr(9) + "STRG+Q")
    EndIf
    Frame3DGadget(#PB_Any, 10, 10, 520,130,"Dia-Show Einstellungen")
    ;
    ; Image Path
    ;
    TextGadget(#PB_Any,30,32,100,20,"Bilder Verzeichnis:");
    StringGadget(#DIASHOW_IMAGE_PATH,130,30,300,20,PREFS_IMAGE_PATH.s);
    ButtonGadget(#DIASHOW_CHOOSE_PATH,432,30,80,20,"Auswählen...");
    GadgetToolTip(#DIASHOW_IMAGE_PATH,"Verzeichnis der anzuzeigenden Bilder");
    GadgetToolTip(#DIASHOW_CHOOSE_PATH,"Klicken, um das Bilder Verzeichnis auszuwählen");
    ;
    ; Display delay
    ;
    TextGadget(#PB_Any,30,62,100,20,"Anzeigedauer:");
    TrackBarGadget(#DIASHOW_DELAY,130,60,300,20,5,60,#PB_TrackBar_Ticks);
    TextGadget(#DIASHOW_DELAY_OUTPUT,432,60,80,20,Str(PREFS_DELAY.l)+" Sekunden")
    GadgetToolTip(#DIASHOW_DELAY,"Auswahl der Anzeigedauer pro Bild in Sekunden (5-60 sek.)");
    SetGadgetState(#DIASHOW_DELAY,PREFS_DELAY.l);
    ;
    ; Logfile
    ;
    TextGadget(#PB_Any,30,92,100,20,"Logdatei:");
    StringGadget(#DIASHOW_LOGFILE,130,90,300,20,PREFS_LOG_FILE.s);
    ButtonGadget(#DIASHOW_CHOOSE_LOGFILE,432,90,80,20,"Auswählen...");
    GadgetToolTip(#DIASHOW_LOGFILE,"Name der Protokoll-Datei. Ist keine Datei angegeben so werden auch keine Logdaten geschrieben.");
    ;
    ; Buttons to save, check file count and abort preferences
    ;
    Frame3DGadget(#PB_Any,10,140,520,60,"");
    ButtonGadget(#DIASHOW_BTN_SAVEPREFS ,30 ,160,150,24,"Einstellungen speichern");
    ButtonGadget(#DIASHOW_BTN_COUNT     ,195,160,150,24,"Zähle Bilder");
    ButtonGadget(#DIASHOW_BTN_ABORT     ,360,160,150,24,"Abbrechen");
    ;
    ; Keyboard shortcuts 
    ;
    AddKeyboardShortcut(0, #PB_Shortcut_Control | #PB_Shortcut_Q, #MENU_CLOSE)
    AddKeyboardShortcut(0, #PB_Shortcut_Control | #PB_Shortcut_A, #MENU_ABOUT)
    AddKeyboardShortcut(0, #PB_Shortcut_Control | #PB_Shortcut_H, #MENU_WEBSITE)
    AddKeyboardShortcut(0, #PB_Shortcut_Escape, #MENU_CLOSE)
      
    Repeat
      EventID = WaitWindowEvent()

      If EventID = #PB_Event_CloseWindow  ; If the user has pressed on the close button
        Quit = 1
      EndIf
      
      If EventID = #PB_Event_Menu         ; Menu selected
        MenuID = EventMenu()
        Select MenuID 
          Case #MENU_CLOSE
            Quit = 1
            
          Case #MENU_ABOUT
            CompileInfo$ = Chr(13)+Chr(13)+"Kompiliert am "+FormatDate("%dd.%mm.%yyyy",#PB_Compiler_Date)+" mit PureBasic "+Str(#PB_Compiler_Version)+Chr(13)+Chr(13)+"OS: "+GetOsInfo(OSVersion());
            MessageRequester("Info","Dia-Show v"+VERSION$+Chr(13)+Chr(13)+"Geschrieben 2011 von Sascha 'SieGeL' Pfalz" + CompileInfo$,#MB_ICONINFORMATION);
            
          Case #MENU_WEBSITE
            RunProgram("http://www.saschapfalz.de");
        EndSelect
      EndIf
      
      If EventID = #PB_Event_Gadget
        Select EventGadget()
            
          Case #DIASHOW_DELAY
            SetGadgetText(#DIASHOW_DELAY_OUTPUT,Str(GetGadgetState(#DIASHOW_DELAY))+" Sekunden")
            
          Case #DIASHOW_CHOOSE_PATH
            FPATH$ = PathRequester("Bitte Bilderverzeichnis auswählen",GetGadgetText(#DIASHOW_IMAGE_PATH));  
            If FPATH$
              If Right(FPATH$,1) = "\"
                FPATH$ = Mid(FPATH$,0,Len(FPATH$)-1);
              EndIf
              SetGadgetText(#DIASHOW_IMAGE_PATH,FPATH$)
            EndIf
            
          Case #DIASHOW_CHOOSE_LOGFILE
            FPATH$ = SaveFileRequester("Bitte Logdatei auswählen",GetGadgetText(#DIASHOW_LOGFILE),"Logdateien|*.log",0);  
            If FPATH$
              If Right(FPATH$,1) = "\"
                FPATH$ = Mid(FPATH$,0,Len(FPATH$)-1);
              EndIf
              SetGadgetText(#DIASHOW_LOGFILE,FPATH$)
            EndIf
            
          Case  #DIASHOW_BTN_ABORT
            Quit = 1
            
          Case  #DIASHOW_BTN_COUNT
            rc = ReadRecDir(GetGadgetText(#DIASHOW_IMAGE_PATH),"*.*",DummyList.s());
            If rc <> #True
              Quit = 1
            EndIf
            MaxItems = ListSize(DummyList.s());
            ClearList(DummyList.s())
            MessageRequester("Info","Im Verzeichnis "+Chr(34)+GetGadgetText(#DIASHOW_IMAGE_PATH)+Chr(34)+" befinden sich "+Str(MaxItems)+" Bilder",#MB_ICONINFORMATION);
          Case  #DIASHOW_BTN_SAVEPREFS
            SavePrefs();
            Quit = 1;
        EndSelect
        
      EndIf
      
    Until Quit = 1  ; Event-Loop
    CloseWindow(0)
  EndIf
  
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 170
; FirstLine = 124
; Folding = -
; EnableUnicode
; EnableXP
; EnableOnError
; UseMainFile = Dia-Show.pb
; Executable = Dia-Show.exe
; CommandLine = /c
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant