;**************************************************************************************************
; Include file contains various procedures to be used in conjunction with PureBasic.
; Written 2011 by Sascha 'SieGeL' Pfalz
;**************************************************************************************************

;--------------------------------------------------------------------------------------------------
; Concatinates a path with a filename and/or a directory
;--------------------------------------------------------------------------------------------------
Procedure.s ConcatPath(Path$,toAdd$,separator$ = "\")
  Define filler$ = "", newPath$ = "";
  If Right(Path$,1) <> "\" And Right(Path$,1) <> "/"
    filler$ = separator$
  Else
    filler$ = ""
  EndIf
  newPath$ = Path$ + filler$ + toAdd$
  ProcedureReturn newPath$
EndProcedure

;--------------------------------------------------------------------------------------------------
; Writes a logfile to have full informations during automated runs
;--------------------------------------------------------------------------------------------------
Procedure WriteToLog(lfile$, logstring$)
  Define mydate$, log$,fd;
  If lfile$ = ""
    ProcedureReturn
  EndIf
  mydate$ = FormatDate("[%dd/%mm/%yyyy %hh:%ii:%ss]",Date())
  log$ = mydate$ + ": " + logstring$;
  fd = OpenFile(99, lfile$)
  If fd > 0
    FileSeek(99,Lof(99));
    WriteStringN(99,log$,#PB_Ascii);
    CloseFile(99);
  EndIf
EndProcedure

;--------------------------------------------------------------------------------------------------
; Function reads given directory and puts all found files in a linked list so we can remove them later
; NOTE: This procedure is recursive reading the given directory!
;--------------------------------------------------------------------------------------------------
Procedure ReadRecDir(Target.s, Pattern.s, List Filenames.s())
  Define dnum,dname$,ext$
  dnum = ExamineDirectory(#PB_Any,Target,Pattern)
  If dnum = 0
    MessageRequester("ERROR","ERROR: Unable To examine directory "+Chr(34)+Target+Chr(34)+" !!!");
    ProcedureReturn #False
  EndIf
  While NextDirectoryEntry(dnum)
    dname$  = DirectoryEntryName(dnum);
    If DirectoryEntryType(dnum) = #PB_DirectoryEntry_Directory
      If dname$ <> "." And dname$ <> ".." 
        ReadRecDir(Target+"\" + DirectoryEntryName(dnum), Pattern.s,Filenames.s())  
      EndIf
    Else
      ext$ = LCase(GetExtensionPart(Target + "\" + dname$));
      If ext$ = "jpg" Or ext$ = "jpeg" Or ext$ = "iff" Or ext$ = "bmp" Or ext$ = "png" Or ext$ = "tif"
        AddElement(Filenames.s())
        Filenames.s() = Target + "\" + dname$
      Else
        ;WriteToLog(Logfile$,"Skipping file "+dname$);
      EndIf
    EndIf
  Wend
  FinishDirectory(dnum)
  ProcedureReturn #True
EndProcedure

;--------------------------------------------------------------------------------------------------
; Converts Big Endian Word value to little endian
; Taken from http://forums.purebasic.com/german/viewtopic.php?f=3&t=13238&hilit=Endian&sid=1a7fe061b077132ea4af8210ac598772
;--------------------------------------------------------------------------------------------------

Procedure BSWAP32(value)
  !mov eax, dword [p.v_value]
  !bswap eax
  ProcedureReturn
EndProcedure

;--------------------------------------------------------------------------------------------------
; Converts Big Endian long value to little endian
; Taken from http://forums.purebasic.com/german/viewtopic.php?f=3&t=13238&hilit=Endian&sid=1a7fe061b077132ea4af8210ac598772
;--------------------------------------------------------------------------------------------------

Procedure.w BSWAP16(value.w)
  !xor eax,eax
  !mov ax, word [p.v_value]
  !rol ax, 8
  ProcedureReturn
EndProcedure

;--------------------------------------------------------------------------------------------------
; Extracts the orientation of a given JPG picture
; Taken from PureBasic Forum, see:
; http://www.purebasic.fr/english/viewtopic.php?f=13&t=15592&start=15
;--------------------------------------------------------------------------------------------------
Procedure GetRotationExif(jpg$)
  Define byteOrder,littleEndian,tifFormat,ifd1,nFields,currentTag,orientation,i,num_champ,exifResult;
  If  OpenFile(0, jpg$)
  ;--> Byte 0 of EXIF begins after JPEG header
  FileSeek(0,12)
  ;--> Bytes 0-1 is word order 18761 ($4949) is Intel and 19789 ($4D4D) is Motorola
  byteOrder = ReadWord(0)
  If byteOrder = $4949
    littleEndian = 1;
  Else
    littleEndian = 0;
  EndIf
  ;Debug "little endian = " + Str(littleEndian) + "|" + Hex(byteOrder)
  ;--> For now I only handle Little Endian
  If byteOrder = $4949 Or byteOrder = $4D4D
    ; --> Bytes 2-3 is TIFF format, it's always 42 ($2A). If not, give up.
    tifFormat = ReadWord(0)
    If littleEndian = 0
      tifFormat = BSWAP16(tifFormat)
    EndIf
    ;Debug "TifFormat = "+Hex(tifFormat)
    ;--> This is always $2A. If not, give up.
    If tifFormat = $2A
      ;Debug "Tiff Format okay"
      ;--> Bytes 4-7 is starting offset for IFD (Image File Directory)
      ifd1 = ReadLong(0)
      If littleEndian = 0
        ifd1 = BSWAP32(ifd1)
      EndIf
      ;--> Move to start of IFD
      FileSeek(0,ifd1 + 12)
      ;--> First 2 bytes of IFD is number of field entries
      nFields = ReadWord(0)
      If littleEndian = 0
        nFields = BSWAP16(nFields)
      EndIf
      ;Debug nFields
      ;--> Loop through all fields to find Date/Time stamp
      For i = 1 To nFields
        ;--> Bytes 0-1 contain the Tag for the field.
        currentTag = ReadWord(0) ;&$FFFF
        If littleEndian =0 
          currentTag = BSWAP16(currentTag);
        EndIf
        currentTag = currentTag & $FFFF;
        Select currentTag
          Case 274 ;orientation
            FileSeek(0,Loc(0) + 6)
            orientation= ReadWord(0)
            If littleEndian = 0
              orientation = BSWAP16(orientation)
            EndIf  
            ReadWord(0)
          Default
            num_champ+1
            ;--> Move to next field. Each field is 12 bytes.
            ;--> currentTag (2 bytes) is current Loc(0) so we add 10
            FileSeek(0,Loc(0) + 10)
        EndSelect
      Next i
      CloseFile(0)
      exifResult = orientation
    Else
      ;--> Wrong format, display Unavailable
      ;Debug "Exif: Wrong format"
      exifResult = 0
    EndIf
  Else
    ;--> Wrong byte order, display Unavailable
      ;Debug "Exif: Wrong byte order"
    exifResult = 0
  EndIf
Else
  ;--> Impossible de lire le fichier
  ;Debug "Exif: Unable to open"
  exifResult = 0
EndIf
  ProcedureReturn exifResult
EndProcedure

;--------------------------------------------------------------------------------------------------
; Returns used Operation system version as text
;--------------------------------------------------------------------------------------------------
Procedure.s GetOsInfo(osid)
  Define retval$
  retval$ = "";
  Select osid
    Case #PB_OS_Windows_NT3_51
      retval$ = "Windows NT 3.5x";
      
    Case #PB_OS_Windows_95
      retval$ = "Windows 95";
      
    Case #PB_OS_Windows_NT_4
      retval$ = "Windows NT 4";
      
    Case #PB_OS_Windows_98
      retval$ = "Windows 98";
      
    Case #PB_OS_Windows_ME
      retval$ = "Windows ME";

    Case #PB_OS_Windows_2000
      retval$ = "Windows 2000"

    Case #PB_OS_Windows_XP
      retval$ = "Windows XP"

    Case #PB_OS_Windows_Server_2003
      retval$ = "Windows Server 2003";
      
    Case #PB_OS_Windows_Vista
      retval$ = "Windows Vista";
      
    Case #PB_OS_Windows_Server_2008
      retval$ = "Windows Server 2008";
      
    Case #PB_OS_Windows_7
      retval$ = "Windows 7";
      
    Default
      retval$ = "Unbekannte Windows Version"
  EndSelect
  ProcedureReturn retval$
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 2
; Folding = --
; EnableAsm
; EnableUnicode
; EnableThread
; EnableXP
; EnableUser
; EnableOnError
; UseMainFile = Dia-Show.pb
; Executable = Dia-Show.exe
; EnableCompileCount = 4
; EnableBuildCount = 1
; EnableExeConstant